/**
******************************************************************************
* @author    Arduino development
* @version  1.0
* @date     13.02.2020
* @brief    Basic example for pH module PH-4502C
It is designed to read pH value.
Software calibration function is included.
*   
*/

#include <Arduino.h>

#define PH_SENSOR_PIN           A0
#define AVG_ARRAY_SIZE          5
#define DELAY_BETWEEN_SAMPLES   20 //ms
#define VOLTAGE                 5  //V
#define VOLTAGE_REF             1023
#define PH_REF                  -5.7
#define CALIB_SAMPLES           7
#define USELESS_SAMPLES         3

#define PH_CALIB_REF            6.86 //pH – used as reference for calibration

uint32_t avgValue = 0;
uint16_t analogValBuf[AVG_ARRAY_SIZE];
uint8_t actualIdx = 0;
bool firstSampling = true;
float sensorVoltage;
float phValue;

float calibVal = 33; //change this value to calibrate
bool calibrationEnabled = false; //auto calibration – change calibVal
bool firstCalibSample = true;
uint8_t numCalibSamples = 0;
uint32_t calibAvg = 0;

void calibration();
void fillBuffer();
void getFloatingAvg();
void sampling();

//function for software calibration – depends on variable calibrationEnabled and constant PH_CALIB_REF
void calibration(){
  for (numCalibSamples = 0; numCalibSamples < CALIB_SAMPLES; numCalibSamples++){
    if(firstCalibSample){
        fillBuffer();
        getFloatingAvg();
        firstCalibSample = !firstCalibSample;
    }
    else{
      if(actualIdx < AVG_ARRAY_SIZE){
        analogValBuf[actualIdx] = analogRead(PH_SENSOR_PIN);
        delay(DELAY_BETWEEN_SAMPLES);
        getFloatingAvg();
        actualIdx++;
      }
      else{
        actualIdx = 0;
        numCalibSamples--;
      }
    }
    if(numCalibSamples >= USELESS_SAMPLES && actualIdx != 0){
      calibAvg += avgValue;
    }
  }
  calibAvg = calibAvg / (CALIB_SAMPLES - USELESS_SAMPLES);
  sensorVoltage = (float)calibAvg * VOLTAGE / VOLTAGE_REF;
  calibVal = -PH_REF * sensorVoltage + PH_CALIB_REF;
  Serial.print("Calib value: ");
  Serial.println(calibVal);
}

//fill whole floating average buffer – it is used only at start (empty buffer) 
void fillBuffer(){
  for (int i = 0; i < AVG_ARRAY_SIZE; i++)
  {
    analogValBuf[i] = analogRead(PH_SENSOR_PIN);
    delay(DELAY_BETWEEN_SAMPLES);
  }
}

//calculate actual value of floating average
void getFloatingAvg(){
    for (int i = 0; i < AVG_ARRAY_SIZE; i++){
      avgValue += analogValBuf[i];
    }
    avgValue = avgValue / AVG_ARRAY_SIZE;
}

//when function is called one sample is grabbed
void sampling(){
  if(firstSampling){
    fillBuffer();
    getFloatingAvg();
    firstSampling = !firstSampling;
  }
  else{
    if(actualIdx < AVG_ARRAY_SIZE){
      analogValBuf[actualIdx] = analogRead(PH_SENSOR_PIN);
      getFloatingAvg();
      actualIdx++;
    }
    else{
      actualIdx = 0;
    }
  }
}

void setup() {
  Serial.begin(9600);
  if(calibrationEnabled){
    calibration();
  }
}

void loop() {
  sampling();
  sensorVoltage = (float)avgValue * VOLTAGE / VOLTAGE_REF;
  phValue = PH_REF * sensorVoltage + calibVal;
  Serial.print("pH value: ");
  Serial.println(phValue);
  delay(1000);
}
